/*
 * (c) Copyright 2002 - 2004 -- Anders Torger
 *
 * This program is open source. For license terms, see the LICENSE file.
 *
 */
#include <math.h>

#include "ambisonics.h"
#include "apui_defs.h"
#include "elog.h"

static double
radians(double degrees)
{
    return degrees / 180.0 * M_PI;
}

int
ambisonics_uhj2bformat_filter_length(int rate,
                                     int *delay)
{
    return VCU_ERR_NOT_IMPLEMENTED;
}

int
ambisonics_generate_uhj2bformat_filter(int input_index, /* W X Y Z ... */
                                       struct bfirfilter *target)
{
    return VCU_ERR_NOT_IMPLEMENTED;
}

int
ambisonics_calculate_gains(int n_inputs, /* W X Y Z R S T ... */               
                           int n_speakers,
                           const double azimuth[],
                           const double elevation[],
                           double gains[])
{
    double *gain, az, el;
    int n, i;

    if (n_inputs < 1 || n_inputs > 9 || n_speakers < 1 || azimuth == NULL ||
        elevation == NULL || gains == NULL)
    {
        return VCU_ERR_INVALID_VALUE;
    }
    /*
      These are the basic decoding equations which will only work well for
      fully symmetric arrays.

      These equations expects that the input W signal is -3dB compared to the
      others, which it is in B-format.
    */
    for (n = 0; n < n_speakers; n++) {
        az = radians(azimuth[n]);
        el = radians(elevation[n]);
        for (i = 0; i < n_inputs; i++) {
            gain = &gains[n * n_inputs + i];
            switch (i) {
            case 0: /* W */ *gain = 0.707;                             break;
            case 1: /* X */ *gain = cos(az) * cos(el);                 break;
            case 2: /* Y */ *gain = sin(az) * cos(el);                 break;
            case 3: /* Z */ *gain = sin(el);                           break;
            case 4: /* R */ *gain = 1.5 * sin(el) * sin(el) - 0.5;     break;
            case 5: /* S */ *gain = cos(az) * sin(2.0 * el);           break;
            case 6: /* T */ *gain = sin(az) * sin(2.0 * el);           break;
            case 7: /* U */ *gain = cos(2.0 * az) * cos(el) * cos(el); break;
            case 8: /* V */ *gain = sin(2.0 * az) * cos(el) * cos(el); break;
            }
            elog(LOGDEBUG, "speaker %d, az %.1f, el %.1f, input %d, gain %.4f",
                 n, azimuth[n], elevation[n], i, *gain);
        }
    }
    return 0;
}
