/*
 * (c) Copyright 2002 - 2004 -- Anders Torger
 *
 * This program is open source. For license terms, see the LICENSE file.
 *
 */
#include <string.h>
#include <math.h>

#include "alignalloc.h"
#include "defs.h"
#include "fftplan.h"
#include "bfirfilter.h"
#include "bfir.h"
#include "log2.h"
#include "elog.h"

/*
static void
scale_bfirfilter(struct bfirfilter *bfirfilter,
		 real_t scale)
{
    int n, i, j;

    for (n = 0; n < bfirfilter->n_rates; n++) {
	for (i = 0; i < bfirfilter->f[n].memseg.n_segments; i++) {
	    for (j = 0;
		 j < bfirfilter->f[n].memseg.seglengths[i] / sizeof(real_t);
		 j++)
	    {
		((real_t *)(bfirfilter->f[n].memseg.segments[i]))[j] *= scale;
	    }
	}
    }
}
*/

static void
preprocess_block(void *block,
		 int block_size,
                 int realsize)
{
    static int maxblocksize = 0;
    static void *buf = NULL;    
    double scaled;
    float scalef;
    void *plan, *ibuf;
    int n;

    if (2 * block_size > maxblocksize) {
        free(buf);
	buf = alignalloc(2 * block_size * realsize);
	maxblocksize = 2 * block_size;
    }
    ibuf = buf;
    plan = fftplan_get_real(log2_get(2 * block_size), false, false, realsize);
    if (realsize == 4) {
        fftwf_execute_r2r(plan, block, ibuf);
        memset(block, 0, 2 * block_size * realsize);
        scalef = 1.0 / (float)(2 * block_size);        
        for (n = 0; n < block_size; n += 4) {
            ((float *)block)[(n<<1)+0] = ((float *)ibuf)[n+0] * scalef;
            ((float *)block)[(n<<1)+1] = ((float *)ibuf)[n+1] * scalef;
            ((float *)block)[(n<<1)+2] = ((float *)ibuf)[n+2] * scalef;
            ((float *)block)[(n<<1)+3] = ((float *)ibuf)[n+3] * scalef;
        }
        ((float *)block)[4] = ((float *)ibuf)[block_size] * scalef;
        ibuf = &((float *)ibuf)[2 * block_size];
        ((float *)block)[5] = ((float *)ibuf)[-1] * scalef;
        ((float *)block)[6] = ((float *)ibuf)[-2] * scalef;
        ((float *)block)[7] = ((float *)ibuf)[-3] * scalef;
        for (n = 4; n < block_size; n += 4) {
            ((float *)block)[(n<<1)+4] = ((float *)ibuf)[-n-0] * scalef;
            ((float *)block)[(n<<1)+5] = ((float *)ibuf)[-n-1] * scalef;
            ((float *)block)[(n<<1)+6] = ((float *)ibuf)[-n-2] * scalef;
            ((float *)block)[(n<<1)+7] = ((float *)ibuf)[-n-3] * scalef;
        }
    } else {
        fftw_execute_r2r(plan, block, ibuf);
        memset(block, 0, 2 * block_size * realsize);
        scaled = 1.0 / (double)(2 * block_size);        
        for (n = 0; n < block_size; n += 4) {
            ((double *)block)[(n<<1)+0] = ((double *)ibuf)[n+0] * scaled;
            ((double *)block)[(n<<1)+1] = ((double *)ibuf)[n+1] * scaled;
            ((double *)block)[(n<<1)+2] = ((double *)ibuf)[n+2] * scaled;
            ((double *)block)[(n<<1)+3] = ((double *)ibuf)[n+3] * scaled;
        }
        ((double *)block)[4] = ((double *)ibuf)[block_size] * scaled;
        ibuf = &((double *)ibuf)[2 * block_size];
        ((double *)block)[5] = ((double *)ibuf)[-1] * scaled;
        ((double *)block)[6] = ((double *)ibuf)[-2] * scaled;
        ((double *)block)[7] = ((double *)ibuf)[-3] * scaled;
        for (n = 4; n < block_size; n += 4) {
            ((double *)block)[(n<<1)+4] = ((double *)ibuf)[-n-0] * scaled;
            ((double *)block)[(n<<1)+5] = ((double *)ibuf)[-n-1] * scaled;
            ((double *)block)[(n<<1)+6] = ((double *)ibuf)[-n-2] * scaled;
            ((double *)block)[(n<<1)+7] = ((double *)ibuf)[-n-3] * scaled;
        }
    }
}

void
bfirfilter_from_raw(const void *filter[],
                    const int lengths[],
                    struct bfirfilter *target,
                    int exclusive_rate_index,
                    int realsize)
{
    int n, i, j, k, offset, foffset;
    bool_t naninf;
    uint8_t *p;

    for (n = 0; n < target->n_rates; n++) {
        if (exclusive_rate_index >= 0 && n != exclusive_rate_index) {
            continue;
        }
	for (j = i = offset = foffset = 0; j < target->f[n].n_blocks; j++) {
	    p = &((uint8_t *)target->f[n].memseg.segments[i])[offset];
	    memset(p, 0, target->f[n].block_size * realsize);
	    if (foffset >= lengths[n]) {
		memset(&p[target->f[n].block_size * realsize], 0,
		       target->f[n].block_size * realsize);
	    } else if (foffset + target->f[n].block_size > lengths[n]) {
		memset(&p[target->f[n].block_size * realsize], 0,
		       target->f[n].block_size * realsize);
		memcpy(&p[target->f[n].block_size * realsize],
                       &((const uint8_t *)filter[n])[foffset * realsize],
		       (lengths[n] - foffset) * realsize);
	    } else {
		memcpy(&p[target->f[n].block_size * realsize],
                       &((const uint8_t *)filter[n])[foffset * realsize],
		       target->f[n].block_size * realsize);
	    }
	    preprocess_block(p, target->f[n].block_size, realsize);

            /* Sanity check */
            naninf = false;
            if (realsize == 4) {
                for (k = 0; k < 2 * target->f[n].block_size; k++) {
                    if (isnan(((float *)p)[k]) || isinf(((float *)p)[k])) {
                        naninf = true;
                        break;
                    }
                }
            } else {
                for (k = 0; k < 2 * target->f[n].block_size; k++) {
                    if (isnan(((double *)p)[k]) || isinf(((double *)p)[k])) {
                        naninf = true;
                        break;
                    }
                }
            }
            if (naninf) {
                elog(LOGERROR, "NaN or Inf value! "
                     "(block %d, foffset %d, offset %d)", j, foffset, offset);
                exit(EXIT_FAILURE);
            }

            
	    offset += 2 * target->f[n].block_size * realsize;
	    if (target->f[n].memseg.seglengths[i] == offset) {
		i++;
		offset = 0;
	    }
	    foffset += target->f[n].block_size;
	}
    }
}

bool_t
bfirfilter_verify_filter_compability(const struct prepfilter *prepf,
                                     const struct bfirfilter bfirf[],
                                     int bfirf_len)
{
    int n, i;
    
    if (prepf == NULL || bfirf == NULL ||
        prepf->n_rates != bfirf[0].n_rates)
    {
        elog(LOGDEBUG, "prepf %p, bfirf %p", prepf, bfirf);
        if (prepf != NULL) {
            elog(LOGDEBUG, "prepf->n_rates: %d", prepf->n_rates);
        }
        if (bfirf != NULL) {
            elog(LOGDEBUG, "bfirf[0].n_rates: %d", bfirf[0].n_rates);
        }
	return false;
    }
    for (n = 0; n < prepf->n_rates; n++) {
	if (prepf->rates[n] != bfirf[0].rates[n]) {
            elog(LOGDEBUG, "rate mismatch");
            return false;
	}
    }
    for (n = 1; n < bfirf_len; n++) {
	if (bfirf[n].n_rates != bfirf[0].n_rates) {
            elog(LOGDEBUG, "rate mismatch");
            return false;
	}
	for (i = 0; i < bfirf[0].n_rates; i++) {
	    if (bfirf[n].rates[i] != bfirf[0].rates[i]) {
                elog(LOGDEBUG, "rate mismatch");
                return false;
	    }
	}
    }
    return true;
}

int
bfirfilter_from_prepf(const struct prepfilter *prepf,
                      struct bfirfilter target[])
{
    const void **filter;
    int n, i;

    if (!bfirfilter_verify_filter_compability(prepf, target,
                                              prepf->n_channels))
    {
        return VCU_ERR_INVALID_VALUE;
    }
    filter = alloca(target->n_rates * sizeof(void *));
    for (n = 0; n < prepf->n_channels; n++) {
        /* this cumbersome assignment is made to avoid a compiler warning,
           const void ** is a bit too complicated for gcc... */
        for (i = 0; i < target->n_rates; i++) {
            filter[i] = prepf->filters[n][i];
        }
        bfirfilter_from_raw(filter, prepf->lengths, &target[n], -1,
                            prepf->realsize);
    }
    return 0;
}
