/*
 * (c) Copyright 2002 - 2003 -- Anders Torger
 *
 * This program is open source. For license terms, see the LICENSE file.
 *
 */
#ifndef _FILTERPRG_H_
#define _FILTERPRG_H_

#include <stdlib.h>
#include <dirent.h>

#include "defs.h"
#include "apui_defs.h"
#include "almusvcu.h"
#include "bfir.h"

struct prepfilter {
    /* the number of sample rates available */
    int n_rates;
    /* which 'n_rates' sample rates, in Hz */
    int rates[FILTERPRG_RATES_MAX];
    /* length in samples for each sample rate */
    int lengths[FILTERPRG_RATES_MAX];
    /* number of channels of the filter, note that all channels must have the
       same length */
    int n_channels;
    /* pointers to the filter coefficients */
    void *filters[FILTERPRG_CHANNELS_MAX][FILTERPRG_RATES_MAX];
    int realsize;
};

/*
 * Scan the directory in 'path' for filter programs. Returns an array of filter
 * programs, the length of it is stored in 'n_programs'. Returns NULL on
 * failure. The returned array is stored in a static buffer which is overwritten
 * in the next call. The array must not be deallocated.
 */
struct filterprg *
filterprg_scan_directory(const char path[],
			 int *n_programs);

bool_t
filterprg_scan_file(const char path[],
                    struct filterprg *fprg);

int
filterprg_load(const struct filterprg *fprg,
	       struct prepfilter *prepf,
	       int delay[], /* may be NULL */
               int channel_list[], /* may be null */
               int base_channel_index,
               int n_channels,
	       const int rates[],
	       int n_rates,
               int realsize,
	       void (*progress_callback)(int channel_index,
					 int n_channels,
					 int from_rate,
					 int to_rate,
                                         bool_t deriving),
               int progress_channel_index_start,
               int progress_n_channels);

int
filterprg_copy(const struct filterprg *fprg,
               const char destname[],
               const char destlongname[],
               const char destpath[]);

const char *
filterprg_prgid_ntoa(prgid_t id);

prgid_t
filterprg_prgid_aton(const char s[]);

void
filterprg_free_prepfilter_contents(struct prepfilter *prepf);

void
filterprg_debug_print_program(const struct filterprg *fprg);

#endif
