/*
 * (c) Copyright 2002 - 2003 -- Anders Torger
 *
 * This program is open source. For license terms, see the LICENSE file.
 *
 */
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <dirent.h>
#include <string.h>

#include "grammar.h"
#include "elog.h"

static char current_filename[PATH_MAX];
static int loglevel = LOGERROR;

const char *
yy_tokname(int token)
{
    static char unknown[100];
    
    switch (token) {
    case EOS:
	return "end of statement (;)";
    case COMMA:
	return "comma (,)";
    case SLASH:
	return "slash (/)";
    case LBRACE:
	return "left brace ({)";
    case RBRACE:
	return "right brace (})";
    case BOOLEAN:
	return "boolean";
    case REAL:
	return "real";
    case STRING:
	return "string";
    case FIELD:
	return "field";
    case -1:
	return "end of file (EOF)";
    case INVALID_INPUT:
        return "invalid input";
    default:
	sprintf(unknown, "UNKNOWN (%d)", token);
	return (const char *)unknown;
    }
}

void
yyrestart(FILE *input_file);

void
yy_restart(const char filename[],
	   FILE *input_file,
           int _loglevel)
{
    loglevel = _loglevel;
    if (filename == NULL) {
	current_filename[0] = '\0';
    } else {
	strncpy(current_filename, filename, PATH_MAX);
	current_filename[PATH_MAX - 1] = '\0';
    }    
    yyrestart(input_file);
    yy_input_mode(false, 0);
}

void
yy_perror(const char format[],
	  ...)
{
    char s[16384];
    va_list ap;

    va_start(ap, format);
    vsnprintf(s, 16384, format, ap);    
    va_end(ap);
    s[16383] = '\0';
    if (current_filename[0] != '\0') {
	elog(loglevel, "In file %s: parse error on line %d: %s",
             current_filename, yylineno, s);
    } else {
	elog(loglevel, "parse error on line %d: %s", yylineno, s);
    }
}

void
yy_pwarning(const char format[],
            ...)
{
    char s[16384];
    va_list ap;

    va_start(ap, format);
    vsnprintf(s, 16384, format, ap);    
    va_end(ap);
    s[16383] = '\0';
    if (current_filename[0] != '\0') {
	elog(loglevel, "In file %s: on line %d: %s",
             current_filename, yylineno, s);
    } else {
	elog(loglevel, "on line %d: %s", yylineno, s);
    }
}

void
yy_perror_tok(int expected_token,
	      int got_token)
{
    yy_perror("unexpected token, expected %s, got %s.",
	      yy_tokname(expected_token), yy_tokname(got_token));
}
