/*
 * (c) Copyright 2002 - 2003 -- Anders Torger
 *
 * This program is open source. For license terms, see the LICENSE file.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include <gpm.h>

#include "textgui.h"
#include "keyinput.h"
#include "elog.h"
#include "almusvcu.h"

#ifndef GPM_W_UP
/*
 * GPM_W_UP is only included in the imwheel-patched version of gpm.
 */
#error gpm does not support scroll wheel (you may need to patch it)
#endif

static bool_t ignore_error = false;

static int
keyin_init(void)
{
    int dummy_pipe[2];
    Gpm_Connect conn;

    memset(&conn, 0, sizeof(conn));
    conn.eventMask = GPM_DOWN;
    
    if (Gpm_Open(&conn, 0) == -1) {
	elog(LOGERROR, "Cannot connect to mouse server. Is gpm running?");
	return -1;
    }

    if (gpm_fd == -2) {
        if (ignore_error) {
            /* we create a dummy pipe which we can return instead, there will
               of course never be any input on it */
            if (pipe(dummy_pipe) == -1) {
                elog(LOGERROR, "Could not create pipe: %s", strerror(errno));
            }
            return dummy_pipe[0];
        }
	elog(LOGERROR, "Must be run in console mode (not in X-windows) for "
             "mouse input to work");
	return -1;
    } else if (gpm_fd < 0) {
	elog(LOGERROR, "Invalid gpm_fd value: %d.\n", gpm_fd);
	return -1;
    }
    
    return gpm_fd;
}

static int
keyin_getkey(int fd,
	     int *repeats)
{
    Gpm_Event event;
    int key;
    
    *repeats = 0;
    Gpm_GetEvent(&event);    
    if ((GPM_BARE_EVENTS(event.type) & ~GPM_DOWN) != 0) {
	elog(LOGERROR, "Unexpected event type");
	return KI_KEY_FAILURE;
    }
    switch (event.buttons) {
    case GPM_B_LEFT:
	key = KI_KEY_LEFT;
	break;
    case GPM_B_RIGHT:
	key = KI_KEY_RIGHT;
	break;
    case GPM_W_UP:
	key = KI_KEY_UP;
	break;
    case GPM_W_DOWN:
	key = KI_KEY_DOWN;
	break;
    default:
	return KI_KEY_IGNORE;
    }
    return key;
}

const char *
vcu_name(void)
{
    return "ki_gpm";
}

static struct keyinput keyin_gpm = {
    keyin_init,
    keyin_getkey
};

void *
vcu_share(void)
{
    return (void *)&keyin_gpm;
}

#define GET_TOKEN(token, error)                                                \
    if (get_config_token(&lval) != token) {                                    \
        elog(LOGERROR, "Parse error: " error );                                \
        return false;                                                          \
    }

bool_t
vcu_preinit(int (*get_config_token)(union vcu_lexval *lexval))
{
    union vcu_lexval lval;
    int tok;

    while ((tok = get_config_token(&lval)) > 0) {
	switch (tok) {
	case VCU_LEXVAL_FIELD:
	    if (strcmp(lval.field, "ignore_error") == 0) {
		GET_TOKEN(VCU_LEXVAL_BOOLEAN, "expected boolean for "
                          "ignore_error field");
		ignore_error = lval.boolean;
		GET_TOKEN(VCU_LEX_EOS, "expected end of statement (;)");
	    }
	    break;
	default:
	    elog(LOGERROR, "Parse error: expected field.");
	    return false;
	}
    }
    return true;
}

#undef GET_TOKEN

