/*
 * (c) Copyright 2002 - 2003 -- Anders Torger
 *
 * This program is open source. For license terms, see the LICENSE file.
 *
 */

/*
 * Resampling through bandlimited interpolation. The format handled is single
 * channel with floating point samples.
 */
#ifndef _RESAMPLE_H_
#define _RESAMPLE_H_

#include "defs.h"

typedef struct _resample_t_ resample_t;

/*
 * Create a resampling filter for the given rates. Returns NULL on failure,
 * with RESAMPLE_ERR* stored in 'error'.
 */
#define RESAMPLE_ERR_INVALID_RATE   1 /* rate < 44100 or > 384000 */
#define RESAMPLE_ERR_GCD_TOO_SMALL  2 /* too small rate common divisor */
#define RESAMPLE_ERR_INTERNAL_ERROR 3 /* internal error (memory allocation) */
#define RESAMPLE_ERR_INVALID_VALUE  4
resample_t *
resample_new(int inrate,
	     int outrate,
             int realsize,
	     int *error);

/*
 * Delete resampling filter.
 */
void
resample_delete(resample_t *rs);

/*
 * Resample using the given resampling filter. There are two ways of providing
 * input and writing output. Either buffers are given with their sizes, or
 * the function pointers for reading and/or writing are set. Values not used
 * must be set to NULL (or 0). If 'write_block' is set, then 'output' must be
 * NULL and 'outsize' 0, the same applies for 'read_block'.
 *
 * If 'insize' and/or 'outsize' are used, the resampling will stop when one of
 * them has been reached. The actual number of output samples is given as the
 * return value. On error (memory allocation or 'write_block' error), -1 is
 * returned.
 *
 * 'read_block' should return the number of samples read, and <= 0 at EOF.
 * 'write_block' should return 'n_samples' on success and any other number on
 * failure.
 */
int
resample(resample_t *rs,
	 void *input,
	 void *output,
	 int insize,
	 int outsize,
	 int (*read_block)(void *inbuf, int n_samples, int realsize),
	 int (*write_block)(void *outbuf, int n_samples, int realsize));

#endif
